/*
 * $Id: filters.c 8 2006-07-07 09:42:18Z cedric $
 *
 * Copyright (c) 2006, Cdric TESSIER
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without 
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. 
 *
 *  Coded by Cdric TESSIER, aka NeZetiC, www.nezetic.info
 * 
 */
#include <stdio.h>
#include <string.h>
#include "getopt.h"
#include "structs.h"
#include "strparse.h"
#include "interface.h"


void process_USR(char *mypayload,char *ip_src, char *ip_dest){
	char uname[387];
	char uaddr[130];
	sscanf(mypayload+8,"%s %s \r\n",uaddr,uname);
	parseString(uname);
	if(!FindifexistUsr(usrs.lst,ip_dest)){
		usrs.lst=InsertUsrFirst(usrs.lst,ip_dest,uaddr,uname);
		if(opt.fcapname==NULL && opt.ncurses==0)
			printf("[New User] %s [%s]\nPseudo: %-30.30s\n",ip_dest,uaddr,uname);
		if(opt.ncurses==1)
			RefreshUsrMenu();
	}else{          
		usrs.lst=FindUsr(usrs.lst,ip_dest);
		if(strcmp(usrs.lst->user->name,"N/A")==0)
			strcpy(usrs.lst->user->name,uname);
		if(strcmp(usrs.lst->user->addr,"N/A")==0)
			strcpy(usrs.lst->user->addr,uaddr);
	}
}               

void process_LST(char *mypayload, char *ip_src, char *ip_dest){
	char cname[387];
	char caddr[130];
	sscanf(mypayload,"LST %s %s \r\n",caddr,cname);
	parseString(cname);
	if(FindifexistUsr(usrs.lst,ip_dest)){
		usrs.lst=FindUsr(usrs.lst,ip_dest);
	}else{          
		usrs.lst=InsertUsrFirst(usrs.lst,ip_dest,"N/A","N/A");
	}               
	if(!FindifexistCnt(usrs.lst->user->contacts,caddr)){
		usrs.lst->user->contacts=InsertCntFirst(usrs.lst->user->contacts);
		strcpy(usrs.lst->user->contacts->contact->addr,caddr);
		strcpy(usrs.lst->user->contacts->contact->name,cname);
		usrs.lst->user->nbrc++;
	}
//	if(opt.fcapname==NULL && opt.ncurses==0)
//		printf("[New Contact] %-30.30s  [%s]\n",cname,caddr);
}

void process_MSG(char *mypayload, char *ip_src, char *ip_dest){
	char uname[387];
	char uaddr[130];
	char cname[387];
	char caddr[130];
	char ip[16];
	char *buf;
	int sens=0;
	if (strstr((const char*)mypayload, "Content-Type: text/plain") != NULL) {
		sscanf(mypayload,"MSG %s %s \r\n",caddr,cname);
		parseString(cname);
		if(strlen(caddr)>6){ // Test le sens 1=from contact, 0=from user
			sens=1;
			strcpy(ip,ip_dest);
		}else{
			strcpy(ip,ip_src);
		}
		if(FindifexistUsr(usrs.lst,ip)){
			usrs.lst=FindUsr(usrs.lst,ip);
			strcpy(uname,usrs.lst->user->name);
			strcpy(uaddr,usrs.lst->user->addr);
		}else{
			strcpy(uname,"N/A");
			strcpy(uaddr,"N/A");
			usrs.lst=InsertUsrFirst(usrs.lst,ip,uaddr,uname);
			if(opt.fcapname==NULL && opt.ncurses==0)
				printf("[New User] %s [%s]\nPseudo: %-30.30s\n",ip,uaddr,uname);
			if(opt.ncurses==1)
				RefreshUsrMenu();
		}
		if(sens==1 && !FindifexistCnt(usrs.lst->user->contacts,caddr)){
			usrs.lst->user->contacts=InsertCntFirst(usrs.lst->user->contacts);
			strcpy(usrs.lst->user->contacts->contact->addr,caddr);
			strcpy(usrs.lst->user->contacts->contact->name,cname);
			usrs.lst->user->nbrc++;
			if(opt.fcapname==NULL && opt.ncurses==0)
				printf("[New Contact] %-30.30s  [%s]\n",cname,caddr);
		}
		handleInstantMessage((u_char*)mypayload);


		if(sens){
			usrs.lst->user->messages=InsertMsgFirst(usrs.lst->user->messages,ip_src,caddr,cname,ip_dest,uaddr,uname,(char*)mypayload);
			if(opt.ncurses==0)
				printf("[New Message] %s -> %s\nFrom: %-30.30s [%s]\nTo:   %-30.30s [%s]\n", ip_src, ip_dest,cname,caddr,uname,uaddr);
		}else{
			usrs.lst->user->messages=InsertMsgFirst(usrs.lst->user->messages,ip_src,uaddr,uname,ip_dest,"N/A","N/A",(char*)mypayload);
			if(opt.ncurses==0)
				printf("[New Message] %s -> %s\nFrom: %-30.30s [%s]\n", ip_src,ip_dest,uname,uaddr);
		}

		usrs.lst->user->nbrm++;

		if(opt.ncurses==1)		
			RefreshMsg();
		else
			printf("%s\n\n",(char*)mypayload);

	}else{ // Bidouille pour retrouver l adresse d un utilisateur dans certains cas
		sscanf(mypayload,"MSG %s %s \r\n",caddr,cname);
		if(strlen(caddr)<6){
			if ((buf = strstr((const char*)mypayload, "TypingUser: ")) != NULL) {
				sscanf(buf+12,"%s\r\n",uaddr);
				if(!FindifexistUsr(usrs.lst,ip_src)){
					usrs.lst=InsertUsrFirst(usrs.lst,ip_src,uaddr,"N/A");
					if(opt.fcapname==NULL && opt.ncurses==0)
						printf("[New User] %s [%s]\nPseudo: %-30.30s\n",ip_src,uaddr,"N/A");
					if(opt.ncurses==1)
						RefreshUsrMenu();

				}else{
					usrs.lst=FindUsr(usrs.lst,ip_src);
					if(strcmp(usrs.lst->user->addr,"N/A")==0)
						strcpy(usrs.lst->user->addr,uaddr);
				}
			}
		}
	}
}

