/*
 * $Id: getopt.c 8 2006-07-07 09:42:18Z cedric $
 *
 * Copyright (c) 2006, Cdric TESSIER
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without 
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. 
 *
 *  Coded by Cdric TESSIER, aka NeZetiC, www.nezetic.info
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <locale.h>
#include <sys/utsname.h>
#include <sys/utsname.h>
#include <langinfo.h> /* for the broken config method, optional */
#include "getopt.h"
#include "version.h"

void printUsage(void){
	char options[]=
		"\n"
		"%s %s\n"
		"Options :\n"
		"\t-i <fxpX> : Interface\n"
		"\t-r <file> : Read from pcap file\n"
		"\t-w <file> : Write pcap dump to file\n"
		"\t-o <file> : Write logs to file\n"
		"\t-c : Console mode (no ncurses)\n"
		"\t-v : Display infos\n"
		"\t-h : Print this\n"
		"\n"
		"Usage : ./%s [Options]\n"
		"\n";
	printf(options,APPNAME,APPVERS,APPNAME_S);
	exit(0);
}

void printMyInfos(void){
	struct utsname buf;

	printf ("\n%s %s\nCopyright (C) 2005-2006 Cdric TESSIER (www.nezetic.info)\n",APPNAME,APPVERS);
	if (uname(&buf)!=-1)
		printf ("Running on %s %s (%s)\n\n", buf.sysname, buf.release, buf.machine);
	printf("This program may be freely redistributed under\nthe terms of the BSD License\n\n");
}


int isUTF8(void){
	char *s;

	if (!setlocale(LC_CTYPE, "")) {
		fprintf(stderr, "Can't set the specified locale! Check LANG, LC_CTYPE, LC_ALL.\n");
		return(0);
	}else{
		if (! strcmp(nl_langinfo(CODESET), "UTF-8")) {
			/* UTF-8 detected */
			return(1);
		}else{
			if ((s = getenv("LC_ALL")) || (s = getenv("LC_CTYPE")) || (s = getenv ("LANG"))) {
				if (strstr(s, "UTF-8"))
					/* UTF-8 strangely detected */
					return(1);
				else
					/* UTF-8 not detected */
					return(0);
			}else{
				/* UTF-8 not detected (getenv failed) */
				return(0);
			}
		}
	}
	return(0);
}

void getOpts(int argc, char **argv){
	int option;
	memset( &opt, 0, sizeof( opt ) );
	opt.interface=NULL;
	opt.fcapname=NULL;
	opt.fdumpname=NULL;
	opt.flogname=NULL;
	opt.ncurses=1;
	
	if(!isUTF8())
		opt.utf8=0;
	else
		opt.utf8=1;

	while( (option = getopt( argc, argv, "chi:o:r:vw:" )) != -1 )
	{
		switch( option )
		{
			case 'c' :
				opt.ncurses=0;
				break;
			case 'h' :
				printUsage();
				break;
			case 'i' :
				opt.interface=optarg;
				printf("Listen to : %s\n\n",opt.interface);
				break;
			case 'o' :
				opt.flogname=optarg;
                                break;

			case 'r' :
				opt.fcapname=optarg;
				printf("Read from : %s\n\n",opt.fcapname);
				break;
			case 'v' :
				printMyInfos();
				exit(0);
				break;
			case 'w' :
				opt.fdumpname=optarg;
				break;
			default : printUsage();
		}
	}
}

