/*
 * $Id: interface.c 8 2006-07-07 09:42:18Z cedric $
 *
 * Copyright (c) 2006, Cdric TESSIER
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without 
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. 
 *
 *  Coded by Cdric TESSIER, aka NeZetiC, www.nezetic.info
 * 
 */
#include <stdlib.h>
#include <string.h>
#include "version.h"
#include "interface.h"
#include "structs.h"
#include "strparse.h"

char *HelpText[] = {
    " Key   Action",
    " ",
    " Up    Scroll list up",
    " Down  Scroll list down",
    " Enter Display user messages",
    "  c    Display user contacts",
    "  i    Display user informations",
    "  h    Display this popup",
    "  q    Close Popup/Exit"
};

void InitCurses()
{
	initscr();
	start_color();
	cbreak();
	init_pair(1,COLOR_WHITE,COLOR_BLACK);
	init_pair(2,COLOR_BLACK,COLOR_WHITE);
	init_pair(3,COLOR_RED,COLOR_WHITE);
	curs_set(0);
	noecho();
	keypad(stdscr,TRUE);
	bkgd(COLOR_PAIR(1));
}       

void clearWin(WINDOW *win){
	int i,j;
	int row,col;
	getmaxyx(win, row, col);
	for(i=0;i<row;i++)
		for(j=0;j<col;j++)
			mvwprintw(win,i,j," ");
}

void printStaticText(WINDOW *bar,char *text[],int size){
	int i=0,pos,rows,cols;
	getmaxyx(bar, rows, cols);
	for(pos=3;pos<rows-1;pos++){
		if(i<size)
			mvwprintw(bar,pos,1,"%s",text[i++]);
	}
}

void printCenter(WINDOW *bar, int row, char *message){
	int rows,cols,len;
	len=strlen(message);
	getmaxyx(bar, rows, cols);
	wbkgd(bar,COLOR_PAIR(2));
	mvwprintw(bar,row,(cols-len)/2,message);
}

void drawTopBar(WINDOW *topbar)
{       
	wbkgd(topbar,COLOR_PAIR(2));
	wprintw(topbar," .: %s %s ::",APPNAME,APPVERS);
	mvwprintw(topbar, 0, COLS-20,".: NeZetiC :: 2005");
}       

void drawMessageBar(WINDOW *bar,char *msg)
{       
	wbkgd(bar,COLOR_PAIR(2));
	wdeleteln(bar);
	mvwprintw(bar,0,0," %s", msg);
	wrefresh(bar);
}

void drawHelpPanel(WINDOW *bar)
{       
	int rows,cols;
	getmaxyx(bar, rows, cols);
	wbkgd(bar,COLOR_PAIR(2));
	box(bar, 0, 0);
	printCenter(bar,1,"Help");
	printStaticText(bar,HelpText,ARRAY_SIZE(HelpText));
}

void drawInfosPanel(WINDOW *bar, char *item)
{
	int rows,cols;

	clearWin(bar);
	getmaxyx(bar,rows,cols);
	wbkgd(bar,COLOR_PAIR(2));
	box(bar, 0, 0);
	
	printCenter(bar,1,"Infos");

	if(item!=NULL){
		usrs.lst=FindUsr(usrs.lst,(char*)item);
		mvwprintw(act_wins[2],3,1,"Ip: %.*s",cols-6,usrs.lst->user->ip);
		mvwprintw(act_wins[2],4,1,"Name: %.*s",cols-8,usrs.lst->user->name);
		mvwprintw(act_wins[2],5,1,"Addr: %.*s",cols-8,usrs.lst->user->addr);
		mvwprintw(act_wins[2],6,1,"Contacts: %d",usrs.lst->user->nbrc);
	}
}

void drawContactsPanel(WINDOW *bar,char *item, int sdeb)
{
	int i=0,pos=3,rows,cols;
	cnt_lst cnt_first;

	clearWin(bar);
	wbkgd(bar,COLOR_PAIR(2));
	box(bar, 0, 0);	
	keypad(bar,TRUE);
	scrollok(bar,TRUE);
	getmaxyx(bar,rows,cols);
	wsetscrreg(bar, 3,rows-2 );

	printCenter(bar,1,"Contacts");

	if(item!=NULL){
		usrs.lst=FindUsr(usrs.lst,(char*)item);
		cnt_first=usrs.lst->user->contacts;
		while(usrs.lst->user->contacts!=NULL){
			if(pos>rows-2)
				break;
			if(i>sdeb-1){
				mvwprintw(bar,pos++,1,"[%.*s]",cols-4,usrs.lst->user->contacts->contact->addr);
				mvwprintw(bar,pos++,1,"%-*.*s",cols-2,cols-2,usrs.lst->user->contacts->contact->name);
				pos++;
			}
			usrs.lst->user->contacts=usrs.lst->user->contacts->next;           
			i++;
		}
		usrs.lst->user->contacts=cnt_first;
	}
}

void drawMessagesPanel(WINDOW *bar,char *item, int sdeb)
{
	int i=0,pos=3,rows,cols;
	int lsize,rsize;
	msg_lst msg_first;

	clearWin(bar);
	wbkgd(bar,COLOR_PAIR(2));
	box(bar, 0, 0);	
	keypad(bar,TRUE);
	scrollok(bar,TRUE);
	getmaxyx(bar,rows,cols);
	lsize=(cols-20)*2/3;
	rsize=(cols-4)/3;
	wsetscrreg(bar, 3,rows-2 );

	printCenter(bar,1,"Messages");

	if(item!=NULL){
		usrs.lst=FindUsr(usrs.lst,(char*)item);
		msg_first=usrs.lst->user->messages;
		if(usrs.lst->user->messages!=NULL){
			do{
				if(usrs.lst->user->messages->prev!=NULL)
					usrs.lst->user->messages=usrs.lst->user->messages->prev;    			
				if(pos>rows-2)
					break;
				if(i>sdeb-1){
					if(pos<rows-1)
						mvwprintw(bar,pos++,1,"From: %-*.*s [%-*.*s]", lsize,lsize,usrs.lst->user->messages->message->from_name,rsize,rsize,usrs.lst->user->messages->message->from_addr);
					if(pos<rows-1)
						mvwprintw(bar,pos++,1,"To:   %-*.*s [%-*.*s]",lsize,lsize,usrs.lst->user->messages->message->to_name,rsize,rsize,usrs.lst->user->messages->message->to_addr);
					if(pos<rows-1)
						mvwprintw(bar,pos,1,"%s", usrs.lst->user->messages->message->text);
					pos+=StrNbrLines(usrs.lst->user->messages->message->text,cols-2)+1;
				}
				i++;
			}while(usrs.lst->user->messages!=msg_first);
		}
		usrs.lst->user->messages=msg_first;
	}
}

void drawConfirmPanel(WINDOW *bar)
{
	wbkgd(bar,COLOR_PAIR(2));
	box(bar, 0, 0);
	
	printCenter(bar,1,"Confirm Exit");
	printCenter(bar,3,"[Y]es [N]o");
}

void CreateUsrMenu(){
	int i;
	usrs.lst=usr_first;
	bg_items = (ITEM **)calloc(usrs.nbru+1, sizeof(ITEM *));
	for(i=0;i<usrs.nbru;i++){
		bg_items[i] = new_item(usrs.lst->user->name, usrs.lst->user->ip);
		usrs.lst=usrs.lst->next;
	}      
	bg_items[usrs.nbru] = (ITEM *)NULL; 
	/* Crate menu */
	bg_menu = new_menu((ITEM **)bg_items);

	/* Create the window to be associated with the menu */
	bgmenu = subwin(stdscr,LINES-2, COLS, 1, 0);
	keypad(bgmenu, TRUE);
	/* Set main window and sub window */
	set_menu_win(bg_menu, bgmenu);
	set_menu_sub(bg_menu, derwin(bgmenu, LINES-4, COLS-2, 1, 1));
	set_menu_format(bg_menu, LINES-5, 1);

	/* Set menu mark to the string " * " */
	set_menu_mark(bg_menu, "> ");

	wbkgd(bgmenu,COLOR_PAIR(1));
	/* Print a border around the main window and print a title */
	box(bgmenu, 0, 0);
	menu_opts_off(bg_menu, O_SHOWDESC); // Dsactive la description 
	/* Post the menu */
	post_menu(bg_menu);
	wrefresh(bgmenu);
	refresh();
	usrs.lst=usr_first;

}


void DeleteUsrMenu(){
	int i,nbr;
	nbr=item_count(bg_menu);
	/* Unpost and free all the memory taken up */
	unpost_menu(bg_menu);
	for(i=0;i<nbr;i++)
		free_item(bg_items[i]);
	free_menu(bg_menu);
	delwin(bgmenu);
}

void RefreshUsrMenu(){
	DeleteUsrMenu();
	CreateUsrMenu();
}

void CreateBars(){
	topbar=subwin(stdscr,1,COLS,0,0);
	messagebar=subwin(stdscr,1,COLS-1,LINES-1,1);

	/* Draw Bars */
	drawTopBar(topbar);
	drawMessageBar(messagebar,"Press h for help");
}

void DeleteBars(){
	delwin(topbar);
	delwin(messagebar);
}

void CreatePanel(int id, int rows, int cols){
	act_wins[id] = newwin(rows, cols,(LINES-rows)/2,(COLS-cols)/2);
        act_panels[id] = new_panel(act_wins[id]);
        set_panel_userptr(act_panels[id], &panel_datas[id]);
        panel_datas[id].hide = TRUE;
        hide_panel(act_panels[id]);
}

void CreatePanels(){
	int rows,cols;

	getmaxyx(stdscr,rows,cols);

	/* Help Panel */
	CreatePanel(0, 15, 35);
	drawHelpPanel(act_wins[0]);

	/* Confirm Panel */
	CreatePanel(1, 5, 20);
	drawConfirmPanel(act_wins[1]);

	/* Infos Panel */
	CreatePanel(2, rows-12, cols-30);
	drawInfosPanel(act_wins[2],NULL);

	/* Contacts Panel */
	CreatePanel(3, rows-8, cols-20);
	drawContactsPanel(act_wins[3],NULL,0);

	/* Messages Panel */
	CreatePanel(4, rows-4, cols-10);
	drawMessagesPanel(act_wins[4],NULL,0);

}

void DeletePanels(){
	int i;
	for(i=0;i<NBRPANELS;i++)
		del_panel(act_panels[i]);
	for(i=0;i<NBRPANELS;i++)
		delwin(act_wins[i]);
}

void RefreshMsg(){
	ITEM * cur;
	cur = current_item(bg_menu);
	pos_menu_cursor(bg_menu);

	if(!isHidden(act_panels[4])){
		drawMessagesPanel(act_wins[4],(char*)item_description(cur),cur_scrl_pos);
		update_panels();
		doupdate();
	}
}

int isHidden(PANEL *pan){
	temp = (PANEL_DATA *)panel_userptr(pan);

	if(temp->hide == TRUE)
		return(1);
	return(0);
}

void HidePanel(PANEL *pan){
	temp = (PANEL_DATA *)panel_userptr(pan);
	hide_panel(pan);
	temp->hide = TRUE;
}

void ShowPanel(PANEL *pan){
	temp = (PANEL_DATA *)panel_userptr(pan);
	show_panel(pan);
	temp->hide = FALSE;
}

void HidePanels(){
	int i;
	for(i=0;i<NBRPANELS;i++){
		if(!isHidden(act_panels[i]))
			HidePanel(act_panels[i]);
	}
	drawMessageBar(messagebar,"Press h for help");
}

void CursesLoop(){
	ITEM *cur;
	int i,key,exit_inter=0;

	do {
		key=getch();
		switch(key){
			case KEY_DOWN:
				menu_driver(bg_menu, REQ_DOWN_ITEM);
				HidePanels();
				break;
			case KEY_UP:
				menu_driver(bg_menu, REQ_UP_ITEM);
				HidePanels();
				break;
			case KEY_NPAGE:
				menu_driver(bg_menu, REQ_SCR_DPAGE);
				HidePanels();
				break;
			case KEY_PPAGE:
				menu_driver(bg_menu, REQ_SCR_UPAGE);
				HidePanels();
				break;
			case 'i':

				cur = current_item(bg_menu);
				pos_menu_cursor(bg_menu);

				if(isHidden(act_panels[2])){
					ShowPanel(act_panels[2]);
					drawMessageBar(messagebar,"Infos");
					drawInfosPanel(act_wins[2],(char*)item_description(cur));
				}else{  
					HidePanel(act_panels[2]);
					drawMessageBar(messagebar,"Press h for help");
				}                                
				break;
			case 'c':

				cur = current_item(bg_menu);
				pos_menu_cursor(bg_menu);

				if(isHidden(act_panels[3])){
					ShowPanel(act_panels[3]);
					drawMessageBar(messagebar,"Contacts");
					drawContactsPanel(act_wins[3],(char*)item_description(cur),0);
					update_panels();
					doupdate();
					i=0;
					do{
						key=wgetch(act_wins[3]);
						switch(key){
							case KEY_UP:
								i--;
								drawContactsPanel(act_wins[3],(char*)item_description(cur),i);
								break;
							case KEY_DOWN:
								i++;
								drawContactsPanel(act_wins[3],(char*)item_description(cur),i);
								break;
						}
					}while(key!='c' && key!='q' && key!=ESCAPE);
					HidePanel(act_panels[3]);
					drawMessageBar(messagebar,"Press h for help");
				}
				break;

			case 'h':
				if(isHidden(act_panels[0])){
					ShowPanel(act_panels[0]);
					drawMessageBar(messagebar,"Help");
				}else{
					HidePanel(act_panels[0]);
					drawMessageBar(messagebar,"Press h for help");
				}
				break;
			case ESCAPE:
				HidePanels();
				break;
			case 'q':
				if(isHidden(act_panels[1])){
					ShowPanel(act_panels[1]);
					drawMessageBar(messagebar,"Exit ?");
					update_panels();
					doupdate();
					do{
						key=wgetch(act_wins[1]);
						switch(key){
							case 'y':
								exit_inter=1;
								break;
							case 'n':
								HidePanel(act_panels[1]);
								drawMessageBar(messagebar,"Press h for help");
								break;
						}
					}while(key!='n' && key!='y');
				}
				break;
			case ENTER:
				cur = current_item(bg_menu);
				pos_menu_cursor(bg_menu);

				if(isHidden(act_panels[4])){
                                        ShowPanel(act_panels[4]);
                                        drawMessageBar(messagebar,"Messages");
                                        drawMessagesPanel(act_wins[4],(char*)item_description(cur),0);
                                        update_panels();
					doupdate();
					cur_scrl_pos=0;
					do{
                                                key=wgetch(act_wins[4]);
                                                switch(key){
                                                        case KEY_UP:
								if(cur_scrl_pos > -1)
									cur_scrl_pos--;
                                                                drawMessagesPanel(act_wins[4],(char*)item_description(cur),cur_scrl_pos);
                                                                break;
                                                        case KEY_DOWN:
                                                                cur_scrl_pos++;
                                                                drawMessagesPanel(act_wins[4],(char*)item_description(cur),cur_scrl_pos);
                                                                break;
                                                }
                                        }while(key!=ENTER && key!='q' && key!=ESCAPE);
                                        HidePanel(act_panels[4]);
                                        drawMessageBar(messagebar,"Press h for help");
                                }
				break;
		}
		update_panels();
		doupdate();
		wrefresh(bgmenu);
	} while (exit_inter!=1);

}
