/*
 * $Id: main.c 6 2006-07-07 09:38:05Z cedric $
 * 
 * Copyright (c) 2006, Cdric TESSIER
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without 
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. 
 *
 *  Coded by Cdric TESSIER, aka NeZetiC, www.nezetic.info
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>
#include <signal.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pcap.h>

#include "lpcap.h"
#include "getopt.h"
#include "interface.h"
#include "structs.h"
#include "filters.h"

#define SNAPLEN 262144

pcap_t *desc = NULL;
pcap_dumper_t *savedesc = NULL;
int pktoffset; /* offset of IP packet within wire packet */

sig_atomic_t foad;

void terminate_on_signal(int s) {
	foad = s;
}


/* 
 * Based on driftnet source code
 * Copyright (c) 2001 Chris Lightfoot. All rights reserved.
 * Email: chris@ex-parrot.com; WWW: http://www.ex-parrot.com/~chris/
 */

/* setup_signals:
 * Set up signal handlers. */
void setupSignals(void) {
	int *p;
	/* Signals to ignore. */
	int ignore_signals[] = {SIGPIPE, 0};
	/* Signals which mean we should quit, killing the display child if
	 * applicable. */
	int terminate_signals[] = {SIGTERM, SIGINT, /*SIGSEGV,*/ SIGBUS, SIGCHLD, 0};
	struct sigaction sa;

	sa.sa_flags = SA_RESTART | SA_NOCLDSTOP;

	for (p = ignore_signals; *p; ++p) {
		memset(&sa, 0, sizeof(sa));
		sa.sa_handler = SIG_IGN;
		sigaction(*p, &sa, NULL);
	}

	for (p = terminate_signals; *p; ++p) {
		memset(&sa, 0, sizeof(sa));
		sa.sa_handler = terminate_on_signal;
		sigaction(*p, &sa, NULL);
	}
}

int getLinklevelHdrLength(int type)
{
	switch (type) {
		case DLT_EN10MB:
			return 14;
		case DLT_SLIP:
			return 16;
		case DLT_SLIP_BSDOS:
			return 24;
		case DLT_NULL:
#ifdef DLT_LOOP
		case DLT_LOOP:
#endif  
			return 4;
		case DLT_PPP:
#ifdef DLT_C_HDLC
		case DLT_C_HDLC:
#endif  
#ifdef DLT_PPP_SERIAL
		case DLT_PPP_SERIAL:
#endif  
			return 4;
		case DLT_PPP_BSDOS:
			return 24;
		case DLT_FDDI:
			return 21;
		case DLT_IEEE802:
			return 22;
#ifdef DLT_ATM_RFC1483
		case DLT_ATM_RFC1483:
			return 8;
#endif
#ifdef DLT_PRISM_HEADER
		case DLT_PRISM_HEADER:
			return 32;
#endif
		case DLT_RAW:
			return 0;
#ifdef DLT_ATM_CLIP
		case DLT_ATM_CLIP:      /* Linux ATM defines this */
			return 8;
#endif
#ifdef DLT_LINUX_SLL
		case DLT_LINUX_SLL:     /* fake header for Linux cooked socket */
			return 16;
#endif
#ifdef DLT_IEEE802_11           /* 802.11 wireless ethernet */
		case DLT_IEEE802_11:
			return 32; /* 20030606 email from Nikhil Bobb */ /*44; */
#endif
		default:;
	}
	return(-1);
}
/*
 * End of drifnet code 
 */

void print_ether_header(struct ether_header *paquet){
	int i;
	for (i=0;i<6;i++){
		printf("%.2x",paquet->ether_shost[i]);
		if (i!=5) printf(":");
	}
	printf(" -> ");
	for (i=0;i<6;i++){
		printf("%.2x",paquet->ether_dhost[i]);
		if (i!=5) printf(":");
	}
	printf(" ");
	printf("Type de trame: 0x%.4x\n",paquet->ether_type);
}


void print_ips(struct ip *paquetip){
	printf("%s",inet_ntoa(paquetip->ip_src));
	printf(" -> ");
	printf("%s",inet_ntoa(paquetip->ip_dest));
	printf("\n");
}

void process_packet(u_char *user, const struct pcap_pkthdr *hdr, const u_char *pkt) {
	struct ip *paquetip; 		/* The IP header */
	struct tcp *paquettcp; 		/* The TCP header */
	u_int size_ip,size_tcp,size_payload;
	char *payload; 			/* Packet payload */
	char *mypayload;
	char ip_src[16];
	char ip_dest[16];
	static int count = 1;                   /* packet counter */

	//printf("\nPacket number %d:\n", count);
	count++;

	if (pkt != NULL)
	{
		if(opt.fdumpname != NULL)
			pcap_dump(user,hdr,pkt);

		paquetip=(struct ip*)(pkt+pktoffset);
		size_ip = IP_HL(paquetip)*4;
		if (size_ip < 20) {printf("   * Invalid IP header length: %u bytes\n", size_ip);return;}

		paquettcp = (struct tcp*)(pkt + pktoffset + size_ip);
		size_tcp = TH_OFF(paquettcp)*4;
		if (size_tcp < 20) {printf("   * Invalid TCP header length: %u bytes\n", size_tcp);return;}

		payload = (char *)(pkt + pktoffset + size_ip + size_tcp);
		size_payload = ntohs(paquetip->ip_len) - (size_ip + size_tcp);
		if ((mypayload = malloc(size_payload + 1)) == NULL) printf("out of memory\n");
		memcpy(mypayload, payload, size_payload);
		mypayload[size_payload] = '\0';
		strncpy(ip_src,inet_ntoa(paquetip->ip_src),15);
		ip_src[15]='\0';
		strncpy(ip_dest,inet_ntoa(paquetip->ip_dest),15);
		ip_dest[15]='\0';

		if(strncmp(mypayload,"USR",3)==0 && strncmp(mypayload+6,"OK",2)==0)
			process_USR(mypayload,ip_src,ip_dest);
		if(strncmp(mypayload,"LST",3)==0)
			process_LST(mypayload,ip_src,ip_dest);
		if(strncmp(mypayload,"MSG",3)==0)
			process_MSG(mypayload,ip_src,ip_dest);

		free(mypayload);
		memset(payload, 0,size_payload);
	}

}

void *process_packet_thread(void *v) {
	if(opt.fcapname==NULL){
		while(!foad)
			pcap_dispatch(desc, 1, process_packet,(u_char *)savedesc);
	}else{
		pcap_dispatch(desc, -1, process_packet, (u_char *)savedesc);
	}
	return NULL;    
}      

int main(int argc,char **argv)
{
	struct bpf_program fp;		/* The compiled filter */
	char filter_exp[] = "tcp && port 1863";	/* The filter expression */
	char errbuf[PCAP_ERRBUF_SIZE];
	bpf_u_int32 mask;		/* Our netmask */
	bpf_u_int32 net;		/* Our IP */
	pthread_t packetth;

	usrs.nbru=0; /* Global variable : User Count */	
	usr_first=usrs.lst;	

	getOpts(argc, argv);
	setupSignals();

	if(opt.fcapname == NULL){
		if(getuid() || getgid()){
			fprintf(stderr,"You must be root\n");
			return(0);
		}
		if(opt.interface == NULL){
			if((opt.interface = (char*)pcap_lookupdev(errbuf))==NULL){
				fprintf(stderr,"Error : %s\n", errbuf);
				return(0);
			}
			printf("Listen to default interface : %s \n\n",opt.interface);
		}
		if (pcap_lookupnet(opt.interface, &net, &mask, errbuf) == -1) {
			fprintf(stderr,"Couldn't get netmask for device %s: %s\n", opt.interface, errbuf);
			net = 0;
			mask = 0;
		}
		if ((desc = pcap_open_live(opt.interface, SNAPLEN, 1, 1000, errbuf))==NULL)
		{
			fprintf(stderr,"Can't open %s\n",opt.interface);
			return(0);
		}
	}else{
		if ((desc = pcap_open_offline(opt.fcapname, errbuf))==NULL)
		{
			fprintf(stderr,"Can't open %s\n",opt.fcapname);
			return(0);
		}
	}

	/* Compile and apply the filter */
	if (pcap_compile(desc, &fp, filter_exp, 1, 0) == -1) {
		fprintf(stderr,"Couldn't parse filter %s: %s\n", filter_exp, pcap_geterr(desc));
		return(0);
	}
	if (pcap_setfilter(desc, &fp) == -1) {
		fprintf(stderr,"Couldn't install filter %s: %s\n", filter_exp, pcap_geterr(desc));
		return(0);
	}

	if((pktoffset=getLinklevelHdrLength(pcap_datalink(desc))) == -1){	
		fprintf(stderr, "Unknown data link type %d", pcap_datalink(desc));
		return(0);
	}

	if(opt.ncurses==1){	
		InitCurses();

		CreateBars();
		CreateUsrMenu();
		CreatePanels();
	}

	if(opt.fdumpname!=NULL)
		savedesc = pcap_dump_open(desc,(char*)opt.fdumpname);

	pthread_create(&packetth, NULL, process_packet_thread, NULL);

	if(opt.ncurses==1){	
		CursesLoop(); /* Enter in curse loop mode : interface.c */
	}else if(opt.fcapname==NULL){ /* If live mode */
		while (!foad)
			sleep(1);
	}

	pthread_cancel(packetth); /* make sure thread quits even if it's stuck in pcap_dispatch */
	pthread_join(packetth, NULL);

	if(opt.ncurses==1){	
		DeleteUsrMenu();
		DeletePanels();	
		DeleteBars();
		endwin();
	}

	if(opt.fdumpname!=NULL){
		pcap_dump_flush(savedesc);
		pcap_dump_close(savedesc);
		printf("Dump to file : %s\n",opt.fdumpname);
	}

	pcap_freecode(&fp);
	pcap_close(desc);


	if(opt.flogname!=NULL){
		SaveLogs(usrs.lst,opt.flogname);
		printf("Write logs to file : %s\n",opt.flogname);
	}

	if(opt.ncurses==1)	
		printMyInfos();

	EmptyUsrlist(usrs.lst);	

	//printf("Bye!\n");

	return(0);
}
