/*
 * $Id: strparse.c 8 2006-07-07 09:42:18Z cedric $
 *
 * Copyright (c) 2006, Cdric TESSIER
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without 
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. 
 *
 *  Coded by Cdric TESSIER, aka NeZetiC, www.nezetic.info
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <iconv.h>
#include <panel.h>
#include "getopt.h"


int StrNbrLines(char *str,int maxlinelen){
        int i,linelen=0,nbr=1,size=strlen(str);
        for(i=0;i<size;i++){
                if((str[i]=='\n') || (linelen>(maxlinelen-1))){
                        nbr++;
                        linelen=0;
                }
                linelen++;
        }
        return(nbr);
}

void parseUrlEncode(char *str){
        int i,j=0,len=strlen(str);
        char buf[len];
        char tmp[3];

        for(i=0;i<len;i++){
                if((int)str[i] == '%'){
                        strncpy(tmp,str+i+1,2);
                        tmp[2]='\0';
			if(isdigit(tmp[0]) && isdigit(tmp[1])){
				buf[j] = (char)strtol( tmp, (char**) NULL, 16);
				i+=2;
			}else
				buf[j]=str[i];
                }else{
                        buf[j]=str[i];
                }
                j++;
        }
        buf[j+1]='\0';
        memset(str,0,j+1);
        memcpy(str,buf,j);
}

void parseCantPrint(char *str){
	int i,j=0,len=strlen(str);
	char buf[len];

	for(i=0;i<len;i++){
		if(isprint(str[i]) != 0){
			buf[j]=str[i];
			j++;
		}else if(str[i]==-23 || str[i]==-24 || str[i]==-32){ /* :: */
			buf[j]=str[i];
			j++;
		}
	}
	buf[j+1]='\0';
	memset(str,0,j+1);
	memcpy(str,buf,j);
}

void parseUTF8toIso (const char * inbuf) {
	iconv_t cd;
	char *outbuf, *outbuf_first;
	size_t inbytesleft, outbytesleft, inbytes;
	char *orig;

	orig=(char*)inbuf;
	inbytesleft = strlen(inbuf);
	outbytesleft = strlen(inbuf);
	inbytes = strlen(inbuf);

	cd = iconv_open ("ISO-8859-1", "UTF-8");
	if (cd == (iconv_t) -1) {
		return;
	}

	outbuf = malloc (outbytesleft+1);
	outbuf_first = outbuf;

	if (iconv (cd, &inbuf, &inbytesleft, &outbuf, &outbytesleft) == -1) {
		free(outbuf_first);
		inbuf=orig;
		iconv_close(cd);
		return;
	}
	iconv_close (cd);
	memset(orig,0,inbytes);
	memcpy(orig,outbuf_first,inbytes);
	orig[inbytes-outbytesleft]='\0';
	free(outbuf_first);

	return ;
}

void parseString(char * inbuf){
	if(opt.utf8==0)
		parseUTF8toIso(inbuf);
	parseUrlEncode(inbuf);
	parseCantPrint(inbuf);
}



/* 
 * shh@thathost.com (Sverre H. Huseby) 
 * msniff.c : 0.1.3
 */

void strAppend(char **dst, const char *src)
{
	int dstlen, nbytes;

	if (src == NULL)
		return;
	if (*dst != NULL)
		dstlen = strlen(*dst);
	else
		dstlen = 0;
	nbytes = dstlen + strlen(src) + 1;
	if (*dst == NULL) {
		if ((*dst = malloc(nbytes)) == NULL)
			printf("out of memory\n");
		**dst = '\0';
	} else
		if ((*dst = realloc(*dst, nbytes)) == NULL)
			printf("out of memory\n");
	strcpy(*dst + dstlen, src);
}

char * formatMessage(const char *s)
{
#define MAX_LINE_LEN 78
	const char *word_start, *word_end;
	char *p;
	char *ret = NULL;
	char *prefix = "  ";
	int prefix_len;
	int curr_line_len = 0;
	int add_len;
	int newline_found;

	prefix_len = strlen(prefix);
	while (*s) {
		newline_found = 0;
		while (*s && isspace(*s)) {
			if (*s == '\r' || *s == '\n')
				newline_found = 1;
			++s;
		}
		if (!*s)
			break;
		word_start = s;
		while (*s && !isspace(*s))
			++s;
		word_end = s;
		add_len = word_end - word_start;

		if (add_len < MAX_LINE_LEN - prefix_len
				&& curr_line_len + add_len > MAX_LINE_LEN)
			newline_found = 1;
		if (newline_found && ret != NULL) {
			strAppend(&ret, "\n");
			strAppend(&ret, prefix);
			curr_line_len = prefix_len;
		}
		if (curr_line_len == 0) {
			strAppend(&ret, prefix);
			curr_line_len = prefix_len;
		}
		if (curr_line_len > prefix_len) {
			strAppend(&ret, " ");
			++curr_line_len;
		}
		if ((p = malloc(add_len + 1)) == NULL)
			printf("out of memory\n");
		strncpy(p, word_start, add_len);
		p[add_len] = '\0';
		strAppend(&ret, p);
		curr_line_len += add_len;
		free(p);
	}
	if (ret == NULL) {
		if ((ret = malloc(1)) == NULL)
			printf("out of memory\n");
		*ret = '\0';
	}
	return ret;
}

/* EOF */

void handleInstantMessage(unsigned char *data){
        char *p;
        char *formatted;
        if ((p = strstr((const char*)data, "\r\n\r\n")) == NULL) {
                printf("couldn't find start of message body\n");
                return;
        }
        parseString(p);
        formatted = formatMessage(p);
	memset(data,0,strlen(formatted)+1);
	memcpy(data,formatted,strlen(formatted));
        free(formatted);
}

