/*
 * $Id: structs.c 8 2006-07-07 09:42:18Z cedric $
 *
 * Copyright (c) 2006, Cdric TESSIER
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice, 
 *   this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without 
 *   specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. 
 *
 *  Coded by Cdric TESSIER, aka NeZetiC, www.nezetic.info
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "structs.h"

msg *msgAllocate(){
        msg *buf=(msg*)malloc(sizeof(msg));
        buf->from_ip[0]='\0';
        buf->from_addr[0]='\0';
        buf->from_name[0]='\0';
	buf->to_ip[0]='\0';
        buf->to_addr[0]='\0';
        buf->to_name[0]='\0';
        buf->text=NULL;
        return buf;
}

void msgDestroy(msg *v){
	free(v->text);
        free(v);
}

msg_lst InsertMsgFirst(msg_lst l,char *from_ip,char *from_addr, char *from_name, char *to_ip, char *to_addr, char *to_name, char *text){
	int len=strlen(text);
        msg_lst c=(msg_lst)malloc(sizeof(msg_lst_p));
        c->message=msgAllocate();
	c->message->text = (char*)malloc(sizeof(char)*(len+1));
	strncpy(c->message->text,text,len);
	c->message->text[len]='\0';
	strcpy(c->message->from_ip,from_ip);
	strcpy(c->message->from_addr,from_addr);
	strcpy(c->message->from_name,from_name);
	strcpy(c->message->to_ip,to_ip);
	strcpy(c->message->to_addr,to_addr);
	strcpy(c->message->to_name,to_name);
        c->next=l;
        c->prev=l;
        if(l!=NULL){
		if(l->prev!=NULL){
			l->prev->next=c;
			c->prev=l->prev;
		}
		l->prev=c;
	}
        return(c);
}

void EmptyMsglist(msg_lst l){
        msg_lst c;
	msg_lst frst=l;
	if(l!=NULL){	
		do{
			c=l;
			if(l->next!=NULL)
				l=l->next;
			msgDestroy(c->message);
			free(c);
		}while(l!=frst);
	}
}

void PrintMsglist(msg_lst l) {
	msg_lst frst=l;
	if(l!=NULL){
		do{
			if(l->prev!=NULL)
				l = l->prev;
			if(l->message->text!=NULL){
				printf("[New Message] %s -> %s\nFrom: %-30.30s [%s]\nTo:   %-30.30s [%s]\n", l->message->from_ip,l->message->to_ip,l->message->from_name,l->message->from_addr,l->message->to_name,l->message->to_addr);
				printf("%s\n\n",l->message->text);
			}
		}while(l!=frst);
	}
}

cnt *cntAllocate(){
        cnt *buf=(cnt*)malloc(sizeof(cnt));
        buf->addr[0]='\0';
        buf->name[0]='\0';
        return buf;
}

void cntDestroy(cnt *v){
        free(v);
}

cnt_lst InsertCntFirst(cnt_lst l){
        cnt_lst c=(cnt_lst)malloc(sizeof(cnt_lst_p));
        c->contact=cntAllocate();
        c->next=l;
        c->prev=NULL;
        if(l!=NULL)
                l->prev=c;
        return(c);
}

void EmptyCntlist(cnt_lst l){
        cnt_lst c;
        while(l!=NULL){
                c=l;
                l=l->next;
                cntDestroy(c->contact);
                free(c);
        }
}

void PrintCntlist(cnt_lst l) {
        while(l!=NULL){
                if(l->contact->addr[0]!=0){
                        printf("%-45.45s  [%s]\n",l->contact->name,l->contact->addr);
                }
                l = l->next;
        }
}

usr *usrAllocate(char *ip){
        usr *buf=(usr*)malloc(sizeof(usr));
        strcpy(buf->ip,ip);
        buf->contacts=NULL;
        buf->messages=NULL;
        return buf;
}

void usrDestroy(usr *v){
        EmptyCntlist(v->contacts);
        EmptyMsglist(v->messages);
        free(v);
}

usr_lst InsertUsrFirst(usr_lst l, char *ip,char *addr,char *name){
        usr_lst c=(usr_lst)malloc(sizeof(usr_lst_p));
        c->user=usrAllocate(ip);
	strncpy(c->user->addr,addr,129);
	strncpy(c->user->name,name,386);
	c->user->nbrc=0;
	c->user->nbrm=0;
        c->next=l;
        c->prev=NULL;
        if(l!=NULL)
                l->prev=c;
	usrs.nbru++;
	usr_first=c;               
        return(c);
}

void EmptyUsrlist(usr_lst l){
        usr_lst c;
        while(l!=NULL){
                c=l;
                l=l->next;
                usrDestroy(c->user);
                free(c);
        }
}

void PrintUsrlist(usr_lst l) {
        int i=0;
        while(l!=NULL){
                printf("\nUser: %s [%s]\nName: %.50s\nContacts: %d\n\n",l->user->ip,l->user->addr,l->user->name,l->user->nbrc);
                PrintCntlist(l->user->contacts);
                l = l->next;
                printf("\n");
                i++;
        }
//        printf("Users : %d\n",i);
}

int FindifexistCnt(cnt_lst l,char *addr){
	cnt_lst deb = l;
        while(l!=NULL){
                if(strcmp(l->contact->addr,addr)==0){
                        return(1);
			l=deb;
		}
                l = l->next;
        }
	l=deb;
        return(0);
}
cnt_lst FindCnt(cnt_lst l,char *addr) {
        while(l!=NULL){
                if(strcmp(l->contact->addr,addr)==0)
                        break;
                l = l->next;
        }
        return(l);
}      

int FindifexistUsr(usr_lst l,char *ip) {
	l=usr_first;
        while(l!=NULL){
                if(strcmp(l->user->ip,ip)==0){
                        return(1);
                }
                l = l->next;
        }
        return(0);
}

usr_lst FindUsr(usr_lst l,char *ip) {
	l=usr_first;
        while(l!=NULL){
                if(strcmp(l->user->ip,ip)==0)
                        break;
                l = l->next;
        }
        return(l);
}

int MsgsNbr(usr_lst l){
	int nbr=0;
	while(l!=NULL){
		nbr+=l->user->nbrm;
		l = l->next;
	}
	return(nbr);
}

int CntsNbr(usr_lst l){
        int nbr=0;
        while(l!=NULL){
                nbr+=l->user->nbrc;
                l = l->next;
        }
        return(nbr);
}

void SaveLogs(usr_lst l, char *fname){
	msg_lst msg_first;
	FILE * f_log;
        
	f_log = fopen(fname, "w");

	fprintf(f_log,"[Statistics]\nTotal Users: %d\nTotal Contacts: %d\nTotal Messages: %d\n\n",usrs.nbru,CntsNbr(l),MsgsNbr(l));

	while(l!=NULL){
		fprintf(f_log,"\n[User]\n");
		
		fprintf(f_log,"User: %s [%s]\nName: %.50s\nContacts: %d\nMessages: %d\n",l->user->ip,l->user->addr,l->user->name,l->user->nbrc,l->user->nbrm);
		fprintf(f_log,"\n[User Contacts]\n");
		
		while(l->user->contacts!=NULL){
			if(l->user->contacts->contact->addr[0]!=0){
				fprintf(f_log,"%-45.45s  [%s]\n",l->user->contacts->contact->name,l->user->contacts->contact->addr);
			}
			l->user->contacts = l->user->contacts->next;
		}
		msg_first=l->user->messages;
		
		fprintf(f_log,"\n[User Messages]\n");

		if(l->user->messages!=NULL){
			do{
				if(l->user->messages->prev!=NULL)
					l->user->messages=l->user->messages->prev;

				fprintf(f_log,"From: %-45.45s [%-s]\n",l->user->messages->message->from_name,l->user->messages->message->from_addr);
				fprintf(f_log,"To:   %-45.45s [%-s]\n",l->user->messages->message->to_name,l->user->messages->message->to_addr);
				fprintf(f_log,"%s\n\n", l->user->messages->message->text);

			}while(l->user->messages!=msg_first);
			l->user->messages=msg_first;
		}
		l = l->next;
		fprintf(f_log,"\n");
	}

        fclose(f_log);
}
